####
## cv.pred.r 
## performs k - fold cross validation
## Janusz 
####

cv.pred.aggr <-function(
         data.dir = NULL, #(TODO: add on-line clustering)
         aggregated.matrix, cluster.labels, patient.files, patient.class, patient.ids,
         cv.k=10,
         method.vector=c("cart"),
         data.source.type=NULL,
         n.features=c(25),
         n.fs=max(n.features),          
         method.fs="none",
         n.pca=n.fs,
         n.iso=0,k.iso=5,
         weighted=FALSE,           
         min.cluster.count=0,
         save.data=FALSE,
         discrete=FALSE,
         data.set.name="spectro_data",
         test_on_train=FALSE,
         rf.fs=FALSE,          
         output.dir = TMP.OUTPUT.DIR
        )
{
      
  if(discrete==TRUE && n.pca>0) {
    warning("Variables will not be discrete after PCA")
    discrete<-FALSE
  }
        
  weka.var.descr<-NULL
  ppc_prob<-NULL
 
  score.matrix.list<-list()
      
  len<-length(patient.files)
  cv.splits<-split(sample(1:len,len),1:cv.k)
 
  for(i in 1:cv.k){
    print(paste("Evaluating fold:",i))
  
      tnum<-cv.splits[[i]]
      train.files<-patient.files[-tnum]
      train.class<-patient.class[-tnum]
      train.ids<-patient.ids[-tnum]
  
      test.files<-patient.files[tnum]
      test.class<-patient.class[tnum]
      test.ids<-patient.ids[tnum]
       
      train.data<-as.matrix(aggregated.matrix[,-tnum])  
      dimnames(train.data)<-list(row.names(aggregated.matrix),variable.names(aggregated.matrix)[-tnum])
      test.data<-as.matrix(aggregated.matrix[,tnum])  
      dimnames(test.data)<-list(row.names(aggregated.matrix),variable.names(aggregated.matrix)[tnum])

## feature selection methods

  ## removing small clusters
    removed.count<-0
    if(min.cluster.count>0){
      print("Removing small clusters...")
      indxs<-find.large.clusters(train.data,min.cluster.count)
      removed.count<-sum(indxs)-nrow(train.data)
      dimnames.train<-dimnames(train.data)
      dimnames.test<-dimnames(test.data)
      train.data<-as.matrix(train.data[indxs,])  
      test.data<-as.matrix(test.data[indxs,])  
      dimnames(train.data)<-list(dimnames.train[[1]][indxs],dimnames.train[[2]])
      dimnames(test.data)<-list(dimnames.test[[1]][indxs],dimnames.test[[2]])
      cluster.lab.i<-cluster.labels[indxs,]
   }else cluster.lab.i<-cluster.labels

#n.fs<-max(n.features)

##
## TODO: pass feature selection method as function
##    

        
 if(method.fs=="aggr"){
        print("Rank aggregation ...")
        res <- rank.peaks(train.data,cluster.lab.i, train.files, train.class,train.ids, output.dir,
                          n.best = n.fs, min.cluster.count=0, n.best.from.ppc=FALSE)
        print("jest res")

        median.order <- res$median.order[1:n.fs]
        p.orders <- res$partial.orders
        ppc.order <- p.orders$ppc[1:n.fs,1]
        mi.order <- p.orders$mi[1:n.fs,1]
        tt.order <- p.orders$tt[1:n.fs,1]
        rf.order <- p.orders$rf[1:n.fs,1]
        
        aggr <- rank.aggr(p.orders, output.dir)
        print("jest aggr")

        mc1.order <- aggr$mc1.order[1:n.fs]
        mc1.kem <- aggr$mc1.kanibal[1:n.fs]
        mc2.order <- aggr$mc2.order[1:n.fs]
        mc2.kem <- aggr$mc2.kanibal[1:n.fs]
        mc3.order <- aggr$mc3.order[1:n.fs]
        mc3.kem <- aggr$mc3.kanibal[1:n.fs]
        mc4.order <- aggr$mc4.order[1:n.fs]
        mc4.kem <- aggr$mc4.kanibal[1:n.fs]
 
        sel.features <- list(ppc=as.character(ppc.order), 
                             mi=as.character(mi.order),
                             tt=as.character(tt.order),
                             rf=as.character(rf.order),
                             median=as.character(median.order),
                             mc1.order=as.character(mc1.order),
                             mc1.kem=as.character(mc1.kem),
                             mc2.order=as.character(mc2.order),
                             mc2.kem=as.character(mc2.kem),
                             mc3.order=as.character(mc3.order),
                             mc3.kem=as.character(mc3.kem),
                             mc4.order=as.character(mc4.order),
                             mc4.kem=as.character(mc4.kem)
                            )
        
        fs.method.names <- c("ppc","mi","tt","rf","median","mc1","mc1_kem","mc2","mc2_kem","mc3","mc3_kem","mc4","mc4_kem")
        method.names <- paste(rep(fs.method.names,each=length(method.vector)),method.vector,sep="_")

  }else error("zla metoda redukcji wymiaru")

    row.names(train.data) <- cluster.lab.i[,1]
    row.names(test.data) <- cluster.lab.i[,1]
    rn <- row.names(train.data)
    print(rn)
    print(paste("Removed clusters:",removed.count))
    
    train.class<-as.matrix(train.class)
    test.class<-as.matrix(test.class)      
    colnames(train.class)<-"class"
    colnames(test.class)<-"class"

    print(paste("PCA ...",n.pca))
    pca.list<-my.pca(train.data,n.pca)
    pca.vals <- pca.list$values
    n.pca.gt.zero <- sum(pca.vals>0)
    pca.vectors <- pca.list$vectors[,1:n.pca.gt.zero]
    pca.train.data <- pca.list$proj.data
    pca.test.data <- (t(pca.list$vectors)%*%(test.data-pca.list$center.vector))

    pca.sel.features <- unique(c(sel.features$ppc[1:70],sel.features$mi[1:70],sel.features$tt[1:70],
	sel.features$rf[1:70]))
    len.tmp <- length(pca.sel.features)
    print(paste("Select PCA ...",n.pca, "unique features:", len.tmp))
    sel.pca.list<-my.pca(train.data[pca.sel.features,],n.pca)
    sel.pca.vals <- sel.pca.list$values
    n.sel.pca.gt.zero <- sum(sel.pca.vals>(0.001*(sum(sel.pca.vals))))
 #   sel.pca.vectors <- sel.pca.list$vectors[,1:n.sel.pca.gt.zero]
    sel.pca.train.data <- sel.pca.list$proj.data
    print("ciagle ok")
    print(pca.sel.features)
    sel.pca.test.data <- (t(sel.pca.list$vectors)%*%(test.data[pca.sel.features,]-sel.pca.list$center.vector))
    print("ciagle ok2")
    for(j in c(1:length(n.features))){     
      nf<-n.features[j]  
      print(paste("Classifying with",nf, "features..."))
      
        for(s in 1:length(fs.method.names)){    
          fs.str <- fs.method.names[s]
          print(paste("using fs method",fs.str))
          sel.row.names <- sel.features[[s]]
          train.data.fs<-as.matrix(train.data[sel.row.names,])
          test.data.fs<-as.matrix(test.data[sel.row.names,])
          rn <- row.names(train.data.fs)
          print(rn)
 
          train.data.nf<- data.frame(t(train.data.fs[1:nf,]),class = train.class)
          test.data.nf<- data.frame(t(test.data.fs[1:nf,]),class =test.class)
      
          model.predictions<-model.pred(train.data.nf,test.data.nf,method.vector,
                                          weka.var.description=weka.var.descr,ppc.prob=ppc.prob)
          if (s==1) nf.score <- cbind(as.vector(test.class),model.predictions)
          else nf.score <- cbind(nf.score,model.predictions )
        }
 
          print(paste("using fs method", "pca"))
          if(nf<= n.pca.gt.zero){
          train.data.nf<- data.frame(t(pca.train.data[1:nf,]),class = train.class)
          test.data.nf<- data.frame(t(pca.test.data[1:nf,]),class =test.class)
      
          model.predictions <- model.pred(train.data.nf,test.data.nf,method.vector,
                                          weka.var.description=weka.var.descr,ppc.prob=ppc.prob)
          }else{
            print("class unknown")
            model.predictions <- matrix("unknown",ncol=length(method.vector),
               nrow=nrow(test.class))       
          }      
#          if (s==1) nf.score <- cbind(as.vector(test.class),model.predictions)
          nf.score <- cbind(nf.score,model.predictions )

      
          print(paste("using fs method", "selected pca"))
          if(nf<= n.sel.pca.gt.zero){
          train.data.nf<- data.frame(t(sel.pca.train.data[1:nf,]),class = train.class)
          test.data.nf<- data.frame(t(sel.pca.test.data[1:nf,]),class =test.class)
      
          model.predictions <- model.pred(train.data.nf,test.data.nf,method.vector,
                                          weka.var.description=weka.var.descr,ppc.prob=ppc.prob)
          }else{
            print("class unknown")
            model.predictions <- matrix("unknown",ncol=length(method.vector),
               nrow=nrow(test.class))
          }
 #         if (s==1) nf.score <- cbind(as.vector(test.class),model.predictions)
          nf.score <- cbind(nf.score,model.predictions)
       dim.tmp <- dim(nf.score)
      l.tmp <- length(method.names)
      print(dim.tmp)
 #     print(l.tmp)
      colnames(nf.score) <- c("true.class", method.names,
                       paste("pca",method.vector,sep="_"),paste("selected_pca", method.vector,sep="_"))

       if(i==1){
          score.matrix.list<-c(score.matrix.list,list(nf.score))
       }else{ 
          score.matrix.list[[j]]<-rbind(score.matrix.list[[j]],nf.score)
       }
       gc()
   } 
 }
    return(score.matrix.list)
}

