####
## cv.report
## Performs cross validation n times and writes results to files 
## Janusz
####
cv.report<-function(data.dir=NULL,output.dir,aggregated.matrix, cluster.labels, patient.files,
                        patient.class,patient.ids,disease,control,
                        n.features=c(25), method.vector="cart",seed=12341,n.times=1,
                        ...){   
  set.seed(seed)
  n.pred<-length(method.vector)
  results<-array(dim=c(n.times,length(n.features),n.pred,4),dimnames=list(NULL,NULL,method.vector,
                                                              c("corr","ppv","sens","spec")))    

## prepare output files

  for(l in 1:length(n.features)){    
    result.file<-paste(output.dir,"/results","_fs_",n.features[[l]],".csv",sep="")
    tmp<-matrix(nrow=0,ncol=(n.pred+1))
    dimnames(tmp)<-list(NULL,c("true.class",method.vector))
    write.table(tmp,file=result.file, sep =",",row.names=FALSE, col.names = TRUE)
  }

  
## cross validation n.times
            
  for(i in 1:n.times){    
  
    cv.pred.res<-cv.pred(data.dir,aggregated.matrix, cluster.labels,patient.files,
                        patient.class,patient.ids,
                        n.features=n.features, 
                        method.vector=method.vector,...)
      
    for(l in 1:length(n.features)){    
      result.file<-paste(output.dir,"/results","_fs_",n.features[l],".csv",sep="")
      write.table(cv.pred.res[[l]],file=result.file,append=TRUE,col.names=FALSE,row.names=TRUE,sep=",")
      cat("\n",file=result.file,append=TRUE)                  

      for(p in 1:n.pred){
        method<-method.vector[p]
        res<-cv.pred.res[[l]]
        results[i,l,p,"corr"]<-mean(res[,1]==res[,method.vector[p]])
        results[i,l,p,"ppv"]<-(sum(res[(res[,1]==res[,method]),method]==disease)/sum(res[,method]==disease)) #PPV TP/TP+FP
        results[i,l,p,"sens"]<-(sum(res[(res[,1]==res[,method]),method]==disease)/sum(res[,1]==disease)) #TP/TP+FN
        results[i,l,p,"spec"]<-(sum(res[(res[,1]==res[,method]),method]==control)/sum(res[,1]==control)) #TN/TN+FP
      } 
   }   
  }   

## report
  report<-list()
  for(l in 1:length(cv.pred.res)){
    tmp<-matrix(nrow=(n.pred),ncol=(9))
    dimnames(tmp)<-list(NULL,c("method","corr","sd_corr","ppv","sd_ppv","sens","sd_sens","spec","sd_spec"))
    for(p in 1:n.pred){
      tmp[p,"method"]<-method.vector[p]
      tmp[p,"corr"]<-mean(results[,l,p,"corr"])
      tmp[p,"sd_corr"]<-sd(results[,l,p,"corr"])
      tmp[p,"ppv"]<-mean(results[,l,p,"ppv"])
      tmp[p,"sd_ppv"]<-sd(results[,l,p,"ppv"])
      tmp[p,"sens"]<-mean(results[,l,p,"sens"])
      tmp[p,"sd_sens"]<-sd(results[,l,p,"sens"])
      tmp[p,"spec"]<-mean(results[,l,p,"spec"])     
      tmp[p,"sd_spec"]<-sd(results[,l,p,"spec"])     
    }
    write.table(data.frame(tmp),file=paste(output.dir,"/report","_fs_",n.features[l],".csv",sep=""),
                col.names=TRUE,row.names=FALSE,sep=",")  
    report<-c(report,list(tmp))
}        
    return(report)    
}
