####
## dim.red 
## dimention reduction and feature ranking methods
## Janusz
####

##
## PCA
##  
  my.pca<-function(x,n.vectors)
  {               
    P <- t(scale(t(x),center=TRUE,scale=FALSE))
    center.vector<-attr(P,"scaled:center")    
    C <- t(P)%*%P
    eigen.deco <- eigen(C)
    V.prim <- eigen.deco$vectors
    V <-P%*%V.prim              
    X.pca<-t(V)%*%P
    n.vectors<-min(n.vectors,nrow(X.pca))    
    return(list(proj.data=X.pca[c(1:n.vectors),],vectors=V[,c(1:n.vectors)],
                values=eigen.deco$values[c(1:n.vectors)], center.vector=center.vector))
  }                  

##
## PPC
##
  my.ppc<- function(data,cluster.labels, patient.class, patient.ids, n.best=nrow(data),n.splits=10,fix.at.one=FALSE){
  mydata<-list(peaklist= data,ytr =as.factor(patient.class),sample.labels=as.vector(patient.ids))
  splits<-my.ppc.splits(mydata,n.splits,fix.at.one=fix.at.one)
  info <- peak.summary(cluster.labels, mydata, splits)
  diff<-abs(info[,"Pr2-Pr1"])
  best<- sort(diff,index.return=TRUE,decreasing=TRUE)
  n<-min(n.best,nrow(data))
  best.ind<-best$ix[1:n]
  best.info<-info[best.ind,]
  #rearange columns and add ranks
  i1.tmp <- dim(cluster.labels)[2]
  i2.tmp <- length(patient.ids)
  best.best <- -abs(best.info[,"Pr2-Pr1"])
  best.uq <- unique(best.best)
  rank.tmp <- cbind(best.uq,rank(best.uq))
  rank.best <- merge(as.matrix(best.best),rank.tmp,by.x=1,by.y=1)[,2]
  best.info <- cbind(best.info[,1:i1.tmp],best.info[,(i1.tmp+6)],rank.best,best.info[,c((i1.tmp+1):(i1.tmp+5),(i1.tmp+7):(i1.tmp+6+i2.tmp))])
  colnames(best.info)[i1.tmp+1] <- "Pr2-Pr1"
  class.probab<-splits$prhat[best.ind,]
  dimnames(class.probab)<-list(NULL,levels(as.factor(patient.class)))
  return(list(best.ind=best.ind, best.info=best.info,breaks=best.info[,"height.split.point"],
        class.probab=class.probab,all.scores=info[,"Pr2-Pr1"],selected.scores=best.info[,"Pr2-Pr1"]))
}

##
## Cluster count
##
find.large.clusters<-function(data, min.cluster.count){
    sum.na<- function(x){sum(x[!is.na(x)])}
    count<-as.vector(apply(data>0,1,sum.na))
    indxs<-(count>=min.cluster.count)
    return(indxs)
}

##
## T Test
##
my.t.test<-function(data,cluster.labels, patient.class, patient.ids, n.best=nrow(data)){
  class<-as.factor(patient.class)
  m1<-apply(data[,class==(levels(class)[1])],1,mean)
  m2<-apply(data[,class==(levels(class)[2])],1,mean)
  var1<-apply(data[,class==(levels(class)[1])],1,var)
  var2<-apply(data[,class==(levels(class)[2])],1,var)

  t<-((m1-m2)/
      sqrt(abs(
      var1/length(class[class==(levels(class)[1])])
      +var2/length(class[class==(levels(class)[2])])
      )))
  #t[i]<-t.test(data[i,class==(levels(class)[1]],data[i,class==(levels(class)[2]])
  st<-sort(abs(t),ind=TRUE,decreasing=TRUE)
  n<-min(n.best,nrow(data))
  best.ind<-st$ix[1:n]
  best.info<-cbind(cluster.labels[best.ind,],t[best.ind],data[best.ind,])

  i1.tmp <- dim(cluster.labels)[2]
  i2.tmp <- length(patient.ids)
  best.best <- -abs(as.double(best.info[,(i1.tmp+1)]))
  best.uq <- unique(best.best)
  rank.tmp <- cbind(best.uq,rank(best.uq))
  rank.best <- merge(as.matrix(best.best),rank.tmp,by.x=1,by.y=1)[,2]
  best.info <- cbind(best.info[,1:i1.tmp],best.info[,(i1.tmp+1)],rank.best,best.info[,(i1.tmp+2):(i1.tmp+1+i2.tmp)])
  dimnames(best.info)<-list(NULL,c(dimnames(cluster.labels)[[2]],"t_test","rank_best",patient.ids))
  return(list(best.ind=best.ind, best.info=best.info,all.scores=t,selected.scores=t[best.ind]))
}

##
## Correlation
##
correlation<-function(data,cluster.labels, patient.class, patient.ids, n.best=nrow(data),method=method){
  
  class<-as.factor(patient.class)

  class<-ifelse(class==(levels(class)[1]),1,2) 
  my.corr<-function(x) return(cor(x,class,method=method))

  c<-apply(data,1,my.corr)


  sc<-sort(abs(c),ind=TRUE,decreasing=TRUE)
  n<-min(n.best,nrow(data))
  best.ind<-sc$ix[1:n]
  best.info<-cbind(cluster.labels[best.ind,],c[best.ind],data[best.ind,])
  dimnames(best.info)<-list(NULL,c(dimnames(cluster.labels)[[2]],"class_corr",patient.ids))
  return(list(best.ind=best.ind, best.info=best.info,all.scores=c, selected.scores=c[best.ind]))
}

##
## Probability estimation
##
within.class.cont.probability<-function(x,class.vect,n.density)
{
        dx=density(x)
        r=range(dx$x)
        rl=r[1]
        ru=r[2]

        data1<-x[class.vect==1]
        data2<-x[class.vect==2]
                
        d1=density(data1,n=n.density,from=rl,to=ru)$y
        d1[d1<=0]<-(min(abs(d1[d1!=0]))/10)
        d2=density(data2,n=n.density,from=rl,to=ru)$y
        d2[d2<=0]<-(min(abs(d2[d2!=0]))/10)
	
        pr1<-d1/sum(d1)
        pr2<-d2/sum(d2)
        return(list(pr1=pr1,pr2=pr2))	
}

within.class.discr.probability<-function(x,class.vect,breaks=breaks,type=7)
{
  if(type==0){
    hist<-hist(x,plot=FALSE,breaks=breaks)
    b<-hist$breaks          
#    }else{
#        alpha <- (1:(breaks/2))/((breaks/2) + 1)
#    
#       #if (ppc.options$debug) cat(i,fill=T)
#        o <- x > 0
#        temp <- sort(x[o])
#        temp2 <- pmax(1, trunc(length(temp) * alpha))          
#        b <- c(0, temp[temp2], max(x))
#       b<-unique(b)
#   }
  }else{
   b<-unique(as.vector(quantile(x,probs=seq(0,1,(1/(breaks/2))),type=type)))
  }
# print(b)
# print(x)
 if (b[1]==0) b<-c((b[1]-diff(b)[1]),b)
        
  data1<-x[class.vect==1]
  data2<-x[class.vect==2]
  
  hist1<-hist(data1,plot=FALSE,breaks=b)
  hist2<-hist(data2,plot=FALSE,breaks=b)
  
  counts1<-hist1$counts+1 #to avoid 0 probability
  counts2<-hist2$counts+1 #to avoid 0 probability
  pr1<-counts1/sum(counts1)
  pr2<-counts2/sum(counts2)
  return(list(pr1=pr1,pr2=pr2,breaks=b,aprio1=(length(data1)/length(x)),aprio2=(length(data2)/length(x))))	
}

##
##  Goodman - Kruskal
##
goodman.kruskal<-function(data,cluster.labels, patient.class, patient.ids, n.best=nrow(data),n.density=512,breaks=10,discr=TRUE)
{
  
  class<-as.factor(patient.class)
  class.vect<-ifelse(class==(levels(class)[1]),1,2) 

  pp<-(sum(class.vect==1)/length(class.vect))
  ptype<-diag(c(pp,(1-pp))) #prawdopodobienstwo klasy
    
  g.k<-function(x){
        if(discr) p1_p2<-within.class.discr.probability(x,class.vect,breaks=breaks)
        else p1_p2<-within.class.cont.probability(x,class.vect,n.density)
        
        p1<-p1_p2$pr1
        p2<-p1_p2$pr2
   	
        p = cbind(p1,p2)%*%ptype # prawdopodobienstwo laczne
	
	pCalk = apply(p,1,sum)  # prawdopodobienstwo poziomu glukozy 1 - sumowanie po wierszach

	pWar = diag(1/pCalk)%*%p # prawdopodobienstwo stanu zdrowia pod warunkiem poziomu glukozy

	V = 1 - apply(pWar^2, 1, sum)
	EV = crossprod(pCalk, V)
	VY = 1 - sum(ptype^2)
	tau = (VY - EV)/VY 
	return(tau)
  }
   
  gk<-apply(data,1,g.k)
  sgk<-sort(abs(gk),ind=TRUE,decreasing=TRUE)
  n<-min(n.best,nrow(data))
  best.ind<-sgk$ix[1:n]
  best.info<-cbind(cluster.labels[best.ind,],gk[best.ind],data[best.ind,])
  dimnames(best.info)<-list(NULL,c(dimnames(cluster.labels)[[2]],"good_krusk",patient.ids))
  return(list(best.ind=best.ind, best.info=best.info,all.scores=gk, selected.scores=gk[best.ind]))  
}

##
## Mutual Information
##
mutual.info<-function(data,cluster.labels, patient.class, patient.ids, n.best=nrow(data),n.density=512,breaks=10,discr=TRUE){
 
  class<-as.factor(patient.class)
  class.vect<-ifelse(class==(levels(class)[1]),1,2) 

  pp<-(sum(class.vect==1)/length(class.vect))
  ptype<-diag(c(pp,(1-pp))) # prawdopodobienstwo klasy
    
  m.i<-function(x){
        if(discr) p1_p2<-within.class.discr.probability(x,class.vect,breaks=breaks)
        else p1_p2<-within.class.cont.probability(x,class.vect,n.density)
        
        p1<-p1_p2$pr1
        p2<-p1_p2$pr2
   	
        p = cbind(p1,p2)%*%ptype # prawdopodobienstwo laczne

  	pCalk = apply(p,1,sum)  # prawdopodobienstwo poziomu glukozy 1 - sumowanie po wierszach
            
        mut<-sum(p*log2(p/(cbind(pCalk,pCalk)%*%ptype)))      
         
	return(mut)
  }
  mi<-apply(data,1,m.i)
  smi<-sort(abs(mi),ind=TRUE,decreasing=TRUE)
  n<-min(n.best,nrow(data))
  best.ind<-smi$ix[1:n]
  best.info<-cbind(cluster.labels[best.ind,],mi[best.ind],data[best.ind,])
    i1.tmp <- dim(cluster.labels)[2]
  i2.tmp <- length(patient.ids)
  best.best <- -abs(as.double(best.info[,(i1.tmp+1)]))
  best.uq <- unique(best.best)
  rank.tmp <- cbind(best.uq,rank(best.uq))
  rank.best <- merge(as.matrix(best.best),rank.tmp,by.x=1,by.y=1)[,2]
  best.info <- cbind(best.info[,1:i1.tmp],best.info[,(i1.tmp+1)],rank.best,best.info[,(i1.tmp+2):(i1.tmp+1+i2.tmp)])
  dimnames(best.info)<-list(NULL,c(dimnames(cluster.labels)[[2]],"mutual_info","rank.best",patient.ids))
  return(list(best.ind=best.ind, best.info=best.info, all.scores=mi, selected.scores=mi[best.ind]))  
}

##
## Bhattacharyya distance
##
bhattacharyya<-function(data,cluster.labels, patient.class, patient.ids, n.best=nrow(data),n.density=512,breaks=10,discr=TRUE)
{
  class<-as.factor(patient.class)
  class.vect<-ifelse(class==(levels(class)[1]),1,2) 

  bhatta<-function(x)
  {
   if(discr) p1_p2<-within.class.discr.probability(x,class.vect,breaks=breaks)
        else p1_p2<-within.class.cont.probability(x,class.vect,n.density)
        
   pr1<-p1_p2$pr1
   pr2<-p1_p2$pr2
   return(-log(sum((pr1^.5)*(pr2^.5))))
  }
  
  bhatta<-apply(data,1,bhatta)

  st<-sort(abs(bhatta),ind=TRUE,decreasing=TRUE)
  n<-min(n.best,nrow(data))
  best.ind<-st$ix[1:n]
  best.info<-cbind(cluster.labels[best.ind,],bhatta[best.ind],data[best.ind,])
  dimnames(best.info)<-list(NULL,c(dimnames(cluster.labels)[[2]],"bhatta_dist",patient.ids))
  return(list(best.ind=best.ind, best.info=best.info,all.scores=bhatta, selected.scores=bhatta[best.ind]))
}

##
## Resistor average distance
##
resistor.average<-function(data,cluster.labels, patient.class, patient.ids, n.best=nrow(data),n.density=512,breaks=10,discr=TRUE)
{
  class<-as.factor(patient.class)
  class.vect<-ifelse(class==(levels(class)[1]),1,2) 
  
  kullback.leibler<-function(x)
  {
   if(discr) p1_p2<-within.class.discr.probability(x,class.vect,breaks=breaks)
        else p1_p2<-within.class.cont.probability(x,class.vect,n.density)
   pr1<-p1_p2$pr1
   pr2<-p1_p2$pr2
   kl1<-sum(pr1*(log2(pr1/pr2)))
   kl2<-sum(pr2*(log2(pr2/pr1)))      
   return(list(kl1=kl1,kl2=kl2)) 
  }
  
  res.avg<-function(x)
  {
   kl<-kullback.leibler(x)
   kl1<-kl$kl1
   kl2<-kl$kl2
   return(1/(1/kl1+1/kl2))
  }
 res.avg<-apply(data,1,res.avg)

 st<-sort(abs(res.avg),ind=TRUE,decreasing=TRUE)
 n<-min(n.best,nrow(data))
 best.ind<-st$ix[1:n]
 best.info<-cbind(cluster.labels[best.ind,],res.avg[best.ind],data[best.ind,])
 dimnames(best.info)<-list(NULL,c(dimnames(cluster.labels)[[2]],"res_avg_dist",patient.ids))
 return(list(best.ind=best.ind, best.info=best.info,all.scores=res.avg, selected.scores=res.avg[best.ind]))
}

##
## Random Forest
##
  
random.forest<-function(data, cluster.labels, patient.class, patient.ids, n.best=nrow(data),mtry0=round(sqrt(nrow(data))), jbt=1000){
  class <- as.factor(patient.class)
  patient.class[class==(levels(class)[1])]=1
  patient.class[class==(levels(class)[2])]=2
  data.m <- t(rbind(data,patient.class))
  nobs <- nrow(data.m)
  nvar <- (ncol(data.m) - 1)
  d <- date()
  r <- runif(1)
  day<-substr(d,9,10)
##tmp rf dir
  if(substr(day,1,1)==" ") day<-paste("0",substr(day,2,2),sep="")
  dd<-paste(substr(d,5,7),day,
            substr(d,23,24),substr(d,12,16),r,sep="_")
  tmp.rf.dir<-paste(RANDOM.FOREST.TMP.DIR,"/","_",dd,"/",sep="") 
  system(paste("mkdir",tmp.rf.dir))
  system(paste("rm ", tmp.rf.dir,"*",sep=""))
###
  write.table(data.m, file=paste(tmp.rf.dir,"train.file.txt",sep=""), col.names=F,row.names=F,quote=F,sep=" ")
# write.matrix(data.m,file=paste(RANDOM.FOREST.TMP.DIR,"train.file2.txt",sep=""))
  cat(paste('"',tmp.rf.dir,"train.file.txt",'"\n','"',tmp.rf.dir,"importance",'"\n',nvar,"\n",nobs,"\n",mtry0,"\n",jbt,"\n","0\n",sep=""),
    file=paste(tmp.rf.dir,"params.txt",sep=""))
  system(paste(RANDOM.FOREST.IMP.PROGRAM, " <", tmp.rf.dir, "params.txt",sep=""))

  rf.out <- read.table(file=paste(tmp.rf.dir,"importance",sep=""), sep="")[,c(1,3)]

  ind <- rf.out[,1]
  rf <- rf.out[,2]
  n <- min(n.best,nrow(data))

  best.ind <- ind[1:n]
  best.info <- cbind(cluster.labels[best.ind,],rf[1:n],data[best.ind,])
  i1.tmp <- dim(cluster.labels)[2]
  i2.tmp <- length(patient.ids)

  tmp.scores <- as.double(best.info[,(i1.tmp+1)])-min(as.double(best.info[,(i1.tmp+1)])) # so 0 is the worst score

  best.best <- -abs(tmp.scores)

  best.uq <- unique(best.best)

  rank.tmp <- cbind(best.uq,rank(best.uq))
  rank.best <- merge(as.matrix(best.best),rank.tmp,by.x=1,by.y=1)[,2]
  best.info <- cbind(best.info[,1:i1.tmp],best.info[,(i1.tmp+1)],rank.best,best.info[,(i1.tmp+2):(i1.tmp+1+i2.tmp)])
  write.matrix(best.info,file=paste(tmp.rf.dir,"best_test",sep=""))
  names <- list(NULL,c(dimnames(cluster.labels)[[2]],"rf_test","rank.best",patient.ids))
  print(names)
  dimnames(best.info)<-names
  system(paste("rm -r", tmp.rf.dir))
  rf.all<-sort(abs(ind),ind=TRUE,decreasing=TRUE)
  return(list(best.ind=best.ind, best.info=best.info, all.scores=rf[rf.all$ix], selected.scores=rf[1:n]))
}
