####
##  fdr.r
##  original code from ppc R package.
##  modified by Janusz.
##  rf added by Marta
###
compute.fdr <- function(data,class.vect, nsplits=10, nperms=100, fix.at.one = FALSE,test="ppc"){
  
  ytr <- class.vect
  nperms <- nperms
  m <- nrow(data)
  
  ttstar <- matrix(NA,nrow=nrow(data),ncol=nperms)

  test.score<-function(ytr){
    
    if(test=="ppc"){    
      mydata<-list(peaklist= data,ytr =as.factor(ytr),sample.labels=paste("p_",c(1:ncol(data),sep=""))) 
      split.fit<- my.ppc.splits(mydata, nsplits)
      t<-abs(split.fit$prhat[,1]-split.fit$prhat[,2])
    }else {
      tmp<-c(1:nrow(data))
      tmp2<-paste("p_",c(1:ncol(data)),sep="")    
      cluster.labels<-cbind(tmp,rep("NA",length(tmp))) 
      colnames(cluster.labels)<-c("cluster_id","mz") 
      if(test=="mi"){
        t <- mutual.info(data,cluster.labels,ytr,tmp2)$best.info[,"mutual_info"]       
      }else if(test=="ra"){
        t <- resistor.average(data,cluster.labels,ytr,tmp2)$best.info[,"res_avg_dist"]       
      }else if(test=="ba"){
        t <- bhattacharyya(data,cluster.labels,ytr,tmp2)$best.info[,"bhatta_dist"]       
      }else if(test=="tt"){
        t <- my.t.test(data,cluster.labels,ytr,tmp2)$best.info[,"t_test"]       
      }else if (test=="rf"){
        t <- random.forest(data,cluster.labels,ytr,tmp2)$best.info[,"rf_test"]
      }
    }    
    return(as.numeric(as.vector(t)))     
  }
  tt<-test.score(ytr)          
  for(i in 1:nperms){
    cat(c("permutation ",i),fill=T)
    ytr2 <- sample(ytr)      
    ttstar[,i] <- test.score(ytr2)
  }  
  return(list(true.score=tt, perm.scores=as.matrix(ttstar)))   
}
  
report.fdr<-function(fdr.res,threshold=NULL){
  if(is.null(threshold)){
    min<-min(fdr.res$true.score)
    max<-max(fdr.res$true.score)
    tmp<-(max-min)/100
   threshold=seq(min,(max+5*tmp),tmp) 
  }
  tt<-fdr.res$true.score
  ttstar<-fdr.res$perm.scores
  nperms<-ncol(ttstar)
  m <- nrow(ttstar)         
  
  nt <- length(threshold)
  npeaks <- rep(NA,nt)
  npeaks0 <- npeaks
  for(i in 1:nt){
    npeaks[i] <- sum(abs(tt)>threshold[i])
    npeaks0[i] <- 0
    for(j in 1:nperms){
       npeaks0[i] <- npeaks0[i]+sum(ttstar[,j]>threshold[i])
    }
  }
 
  fdr <- (npeaks0/nperms)/npeaks
  q1 <- quantile(abs(tt), .25)
  q2 <- quantile(abs(tt), .75)
  
  pi0 <- min((sum(abs(ttstar)> q1 & abs(ttstar)< q2)/nperms)/(.5*m) ,1 )
  
  fdr2 <- fdr*pi0
  
  results <- cbind(threshold, npeaks,npeaks0,fdr,fdr2)
  
  dimnames(results) <- list(NULL,c("threshold", "npeaks","npeaks_in_permuted_trials","fdr","fdr2"))
  return(list(results=results,pi0=pi0, threshold=threshold))
}
