####
## model.pred.r
## classification methods 
## Janusz
## TODO: Take vector of functions as parameter.
####


model.pred<-function(train.data,test.data,method.vector,weka.var.description=NULL,ppc.prob=NULL){

pred.matrix<-matrix(ncol=length(method.vector),nrow =(dim(test.data)[1]))
dimnames(pred.matrix)<-list(NULL,method.vector)

##
## Random Forest 
##
if(is.element("rf",method.vector)){
  rf<-randomForest(class ~ .,data=train.data,ntree=500)
  rf.pred<-predict(rf,test.data)
  pred.matrix[,"rf"]<-as.vector(rf.pred)
}

##
## LDA
##
if(is.element("lda",method.vector)){
  lda<-lda(class ~ .,data=train.data)
  lda.pred<-predict(lda,test.data)$class
   pred.matrix[,"lda"]<-as.vector(lda.pred)
}

##
## QDA
##
if(is.element("qda",method.vector)){
  qda<-qda(class ~ .,data=train.data)
  qda.pred<-predict(qda,test.data)$class
  pred.matrix[,"qda"]<-as.vector(qda.pred)
}

##
##CART
##
if(is.element("cart",method.vector)){
 # tree.par<-tree.control(nobs=length(train.class),minsize=4,mincut= 2)
  cart<-tree(class ~ .,data=train.data)
  cart.pred<-predict(cart,test.data,type = "class")
  pred.matrix[,"cart"]<-as.vector(cart.pred)
}
if(is.element("cart2",method.vector)){
 # tree.par<-tree.control(nobs=length(train.class),minsize=4,mincut= 2)
  cart2<-rpart(class ~ .,data=train.data)
  cart2.pred<-predict(cart2,test.data,type = "class")
  pred.matrix[,"cart2"]<-as.vector(cart2.pred)
}

##
## SVM
##
if(is.element("svm",method.vector)){
  svm<-svm(class ~ .,data=train.data)
  svm.pred<-predict(svm,test.data)
  pred.matrix[,"svm"]<-as.vector(svm.pred)
}

##
## Naive Bayes
##
#if(is.element("nb",method.vector)){
#  nb<-naive.bayes(train.data[,!(dimnames(train.data)[[2]]=="class")],as.vector(train.data[,"class"]))
#  nb.pred<-nb.predict(nb,test.data[,!(dimnames(test.data)[[2]]=="class")])
#  pred.matrix[,"nb"]<-as.vector(nb.pred)
#}
#


##
## Nearest Shrunken Centroids
##
## Original code Tibshirani et. al. R ppc package
if(is.element("shrunk",method.vector)){
#    if(metric == "euclidean") {
      test.matrix<-t(data.matrix(test.data[,!(dimnames(test.data)[[2]]=="class")]))
      dist<-matrix(ncol=ncol(ppc.prob),nrow=ncol(test.matrix))
      for(k in 1:ncol(ppc.prob)){
        marg.dist<- (test.matrix - matrix(ppc.prob[,k], ncol =ncol(test.matrix), nrow = nrow(test.matrix)))^2
        dist[,k] <- apply(marg.dist,2,mean)
      }      
#    }
## could also use median above (instead of mean)    
#    if(metric == "absolute") {
#      test.matrix<-t(data.matrix(test.data[,-(dimnames(test.data)[[2]]=="class")]))
#      n.test<-ncol(test.matrix)
#      for(k in 1:ncol(ppc_prob){
#        marg.dist<- (test.matrix - matrix(ppc_prob[,k], ncol = n.test, nrow = nrow(test.matrix)))
#        dist[,k] <- apply(marg.dist,2,summ)
#      }
#  }
## binomial??      
#    if(metric == "binomial") {
#      
#      ## compute Bayes estimate of pr, to keep it away from 0 or 1
#      ntemp <- matrix(n.class,nrow=nrow(pr),ncol=length(n.class), byrow=T)
#      
#      prbayes<- (pr*ntemp+1)/(ntemp+2)
#      
#      for(k in 1:K){
#        prmat<-matrix(prbayes[,k], ncol = n, nrow = nrow(yhat))
#        temp<- -1*(yhat*log(prmat)+(1-yhat)*log(1-prmat))
#        dd[,k] <- apply(temp,2,summ)
#      }
#      
#   }
#   
  which.is.min <- function(x) {
    y <- seq(length(x))[x == min(x, na.rm=TRUE)]
    y <- y[!is.na(y)]
    if(length(y) > 1)
    y <- sample(y, 1)
    y
  } 
   class.vect<-dimnames(ppc.prob)[[2]]
   shrunk.pred.indx<-apply(dist,1,which.is.min)
   shrunk.pred<-class.vect[shrunk.pred.indx]
   pred.matrix[,"shrunk"]<-as.vector(shrunk.pred)
}

##
## WEKA Naive Bayes - requires weka.jar
##
if(is.element("weka.nb",method.vector)){
  
  cat(weka.var.description,file =paste(TMP.WEKA.DIR,"/train.arff", sep=""))
  cat(weka.var.description,file =paste(TMP.WEKA.DIR,"/test.arff", sep=""))
  
  write.table(train.data,file =paste(TMP.WEKA.DIR,"/train.arff", sep=""),sep=",",col.names=FALSE,row.names=FALSE,quote=FALSE,append=TRUE)
  write.table(test.data,file =paste(TMP.WEKA.DIR,"/test.arff", sep=""),sep=",",col.names=FALSE,row.names=FALSE,quote=FALSE,append=TRUE)

  command<-paste('java -classpath /home/dzanoos/Weka/weka-3-4-4/weka.jar',' weka.classifiers.bayes.NaiveBayes',' -t ', 
    	TMP.WEKA.DIR,'/train.arff ',' -T ',TMP.WEKA.DIR,'/test.arff ',' -p 0',
	' >',TMP.WEKA.DIR,'/nb.results.csv',sep="")
  print("Classifing with Weka ...")  
  system(command)  
  pred.matrix[,"weka.nb"]<-as.matrix(read.table(paste(TMP.WEKA.DIR,"/nb.results.csv",sep=""))[2])
}

##
## WEKA TAN - requires weka.jar
##
if(is.element("weka.tan",method.vector)){
  
  cat(weka.var.description,file =paste(TMP.WEKA.DIR,"/train.arff", sep=""))
  cat(weka.var.description,file =paste(TMP.WEKA.DIR,"/test.arff", sep=""))
  
  write.table(train.data,file =paste(TMP.WEKA.DIR,"/train.arff", sep=""),sep=",",col.names=FALSE,row.names=FALSE,quote=FALSE,append=TRUE)
  write.table(test.data,file =paste(TMP.WEKA.DIR,"/test.arff", sep=""),sep=",",col.names=FALSE,row.names=FALSE,quote=FALSE,append=TRUE)
   
  command<-paste('java -classpath /home/dzanoos/Weka/weka-3-4-4/weka.jar',' weka.classifiers.bayes.BayesNet -D -B ',' -t ', 
    	TMP.WEKA.DIR,'/train.arff ',' -T ',TMP.WEKA.DIR,'/test.arff ',' -p 0 ',
        ' -Q weka.classifiers.bayes.net.search.local.TAN -E weka.classifiers.bayes.net.estimate.SimpleEstimator -A 0.5  ',  
	'>',TMP.WEKA.DIR,'/tan.results.csv',sep="")
  print("Classifing with Weka ...")  
  system(command)  
  pred.matrix[,"weka.tan"]<-as.matrix(read.table(paste(TMP.WEKA.DIR,"/tan.results.csv",sep=""))[2])
}
 
 return(pred.matrix)
}
