####
## peak.info.r
## peak statistics and graphics
## Janusz
####

peak.stats<-function(aggregated.matrix,cluster.labels,patient.files,patient.class,patient.ids,output.dir,
                     n.best=50,n.plot=25,colors=c("red","lightgray"),
                     min.cluster.count=2, ret.lab=FALSE,seperate.output=F)
{
 
  indx<-find.large.clusters(aggregated.matrix, min.cluster.count)
  cut.aggregated.matrix<-aggregated.matrix[indx,]
  cut.cluster.labels<-as.matrix(cluster.labels[indx,])
  dimnames(cut.cluster.labels)<-list(dimnames(cluster.labels)[[1]][indx],dimnames(cluster.labels)[[2]])

  while(!(dev.cur()==1)) dev.off()
      
if(!seperate.output) pdf(paste(output.dir,"/stats.pdf",sep=""))
  
## ppc  
  ppc.stat<-my.ppc(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,1000)
  ppc.peak.info<-ppc.stat$best.info
  ppc.peak.info[is.na(ppc.peak.info)]<-0  
  write.matrix(ppc.peak.info,file =paste(output.dir,"/ppc.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(ppc.peak.info[1:n.best,"Pr2-Pr1"]),main="Peak Probability Contrast")
  ppc.first.data.col<-( dim(ppc.peak.info)[2] - length(patient.class) + 1 )
  
  if(seperate.output) pdf(paste(output.dir,"/ppc.pdf",sep=""))
  plot.hist(ppc.peak.info[,ppc.first.data.col:dim(ppc.peak.info)[2]],ppc.peak.info[,2:3],patient.class,type="ppc",
  ppc.split=ppc.peak.info[,"height.split.point"],title="PPC",colors=colors,test.name="ppc",
  test.val=ppc.peak.info[,"Pr2-Pr1"][1:25],  ret.lab=ret.lab)
  gc()
## tt
  t.stat<-my.t.test(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,n.best)
  t.peak.info<-t.stat$best.info
  write.matrix(t.peak.info,file =paste(output.dir,"/t.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(t.peak.info[,"t_test"]),main="T Test")
  first.data.col<-( dim(t.peak.info)[2] - length(patient.class) + 1 )
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/tt.pdf",sep="")) } 
  plot.hist(t.peak.info[,first.data.col:dim(t.peak.info)[2]],t.peak.info[,2:3],patient.class,title="T Test",colors=colors, 
    test.name="t", test.val=t.peak.info[,"t_test"][1:25])
  gc()
## ra  
  ra.stat<-resistor.average(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,n.best)
  ra.peak.info<-ra.stat$best.info
  write.matrix(ra.peak.info,file =paste(output.dir,"/ra.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(ra.peak.info[,"res_avg_dist"]),main="Resistor Average Distance")
  first.data.col<-( dim(ra.peak.info)[2] - length(patient.class) + 1 )
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/ra.pdf",sep=""))}  
  plot.hist(ra.peak.info[,first.data.col:dim(ra.peak.info)[2]],ra.peak.info[,2:3],patient.class,title=
    "Resistor Average Distance", colors=colors, test.name="ra", test.val=ra.peak.info[,"res_avg_dist"][1:25])
  gc()
## bch
  bch.stat<-bhattacharyya(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,n.best)
  bch.peak.info<-bch.stat$best.info
  write.matrix(bch.peak.info,file =paste(output.dir,"/bch.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(bch.peak.info[,"bhatta_dist"]),main="Bhattacharyya Distance")
  first.data.col<-( dim(bch.peak.info)[2] - length(patient.class) + 1 )
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/bch.pdf",sep="")) } 
  plot.hist(bch.peak.info[,first.data.col:dim(bch.peak.info)[2]],bch.peak.info[,2:3],patient.class,title=
   "Bhattacharyya Distance", colors=colors,test.name="bch", test.val=bch.peak.info[,"bhatta_dist"][1:25])
  gc()
## p.corr
  p.corr.stat<-correlation(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,n.best,method="pearson")
  p.corr.peak.info<-p.corr.stat$best.info
  write.matrix(p.corr.peak.info,file =paste(output.dir,"/corr_pears.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(p.corr.peak.info[,"class_corr"]),main="Pearson Correlation")
  first.data.col<-( dim(p.corr.peak.info)[2] - length(patient.class) + 1 )
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/p.corr.pdf",sep=""))}  
  plot.hist(p.corr.peak.info[,first.data.col:dim(p.corr.peak.info)[2]],p.corr.peak.info[,2:3],patient.class,title=
    "Pearson Correlation", colors=colors, test.name="p_cor", test.val=p.corr.peak.info[,"class_corr"][1:25])
  gc()
## s.corr
  s.corr.stat<-correlation(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,n.best,method="spearman")
  s.corr.peak.info<-s.corr.stat$best.info
  write.matrix(s.corr.peak.info,file =paste(output.dir,"/corr_spear.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(s.corr.peak.info[,"class_corr"]),main="Spearman Correlation")
  first.data.col<-( dim(s.corr.peak.info)[2] - length(patient.class) + 1 )
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/s.corr.pdf",sep=""))}  
  plot.hist(s.corr.peak.info[,first.data.col:dim(s.corr.peak.info)[2]],s.corr.peak.info[,2:3],patient.class,title=
    "Spearman Correlation", colors=colors, test.name="s_cor", test.val=s.corr.peak.info[,"class_corr"][1:25])
  gc()                      
## gk.stat  
  gk.stat<-goodman.kruskal(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,n.best)   
  gk.peak.info1<-gk.stat$best.info
  write.matrix(gk.peak.info1,file =paste(output.dir,"/gk.discr.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(gk.peak.info1[,"good_krusk"]),main="Goodman-Kruskal")
  first.data.col<-( dim(gk.peak.info1)[2] - length(patient.class) + 1 )
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/gk.pdf",sep=""))}  
  plot.hist(gk.peak.info1[,first.data.col:dim(gk.peak.info1)[2]],gk.peak.info1[,2:3],patient.class,title=
    "Goodman-Kruskal",colors=colors,test.name="gk", test.val=gk.peak.info1[,"good_krusk"][1:25])
  gc()
                       
## gk.cont
#  gk.peak.info2<-goodman.kruskal(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,1000,
#     discr=FALSE)$best.info   
#  write.matrix(gk.peak.info2,file =paste(output.dir,"/gk.cont.best.peak.info.csv", sep=""),sep=",")
#  gc()
#  x11()
#  plot(abs(gk.peak.info2[,4]))
                       
## mi
  mi.stat<-mutual.info(cut.aggregated.matrix,cut.cluster.labels,patient.class,patient.files,n.best)
  mi.peak.info<-mi.stat$best.info
  write.matrix(mi.peak.info,file =paste(output.dir,"/mi.cont.best.peak.info.csv", sep=""),sep=",")
  #plot(abs(mi.peak.info[,"mutual_info"]),main="Mutual Information")
  first.data.col<-( dim(mi.peak.info)[2] - length(patient.class) + 1 )
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/mi.pdf",sep=""))}  
  plot.hist(mi.peak.info[,first.data.col:dim(mi.peak.info)[2]],mi.peak.info[,2:3],patient.class,title=
   "Mutual Information",colors=colors, test.name="mi", test.val=mi.peak.info[,"mutual_info"][1:25])
  gc()
  class<-as.factor(patient.class)
  col<-ifelse(class==(levels(class)[1]),colors[1],colors[2]) 
 
## tests comparison plot
  d.test<-data.frame(ppc=ppc.stat$all.scores,mi=mi.stat$all.scores,ra=ra.stat$all.scores,gk=gk.stat$all.scores,
                     t=t.stat$all.scores,p_cor=p.corr.stat$all.scores,s_cor= s.corr.stat$all.scores)                     
    panel.cor <- function(x, y, digits=2, prefix="")
      {
          usr <- par("usr"); on.exit(par(usr))
          par(usr = c(0, 1, 0, 1))
          r <- abs(cor(x, y, method="spearman"))
          txt <- round(r,2)
          txt <- paste(prefix, txt, sep="")
          text(0.5, 0.5, txt)
      }
    pch.plot<-function(x,y) plot(cbind(x,y),pch=".")

    my.panel.smooth<-function(...) points(...,pch=".")
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/pairs.pdf",sep="")) } 
  pairs(as.matrix(abs(d.test)),lower.panel = my.panel.smooth,upper.panel=panel.cor)
    
## PCA
  pca.list<-my.pca(aggregated.matrix,min(n.best,ncol(aggregated.matrix)))
  x<-pca.list$values
  y<-x/sum(x)                                         
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/pca_var.pdf",sep=""))}  
  plot(y,main="PCA Residual Variances")
  data<-t(pca.list$proj.data)
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/pca.pdf",sep=""))}  
  plot(data.frame(data)[,1:5],col=col,main="PCA Dimentions")      

#  biplot(princomp(data[,1;5], cor = T), pc.biplot = T, cex = 0.7, expand = 0.8)

## ISOMAP
  k<-10
  points<-k.isomap(t(aggregated.matrix),k,min(n.best,ncol(aggregated.matrix)))  
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/isomap.pdf",sep=""))}  
  plot(data.frame(points)[,1:5],col=col,main="Isomap Dimentions")        
 
## Random Forest 
  train.data<-data.frame(t(ppc.peak.info[,ppc.first.data.col:dim(ppc.peak.info)[2]]),class=as.matrix(patient.class))  
  colnames(train.data)<-c(paste("cluster_",c(1:length(ppc.peak.info[,2])),"_mz_",round(ppc.peak.info[,2],4),sep=""),"class")
   
  rf<-randomForest(class~.,train.data,proximity=TRUE,importance=TRUE)
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/rf_var.pdf",sep=""))}  
  rf.features<-varImpPlot(rf)
  rf.feat.sort<-sort(rf.features[,1],index.return=TRUE,decreasing=TRUE)
  rf.peak.info<-ppc.peak.info[rf.feat.sort$ix,1:3]
  rf.peak.info<-cbind(rf.peak.info,as.vector(rf.feat.sort$x))
  write.matrix(rf.peak.info,file=paste(output.dir,"/rf.best.peak.info.csv", sep=""),sep=",")
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/rf_mds1.pdf",sep=""))}  
  cords<-MDSplot(rf,train.data$class,pallette=colors,main=paste("RF (oob err %",rf$err.rate[500,1],")",sep=""))
  text(cords$points[,1],cords$points[,2],lab=as.numeric(patient.ids))

  rf2<-randomForest(t(ppc.peak.info[,ppc.first.data.col:dim(ppc.peak.info)[2]]),proximity=TRUE)
## varImpPlot(rf2)
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/rf_mds2.pdf",sep=""))}  
  cords<-MDSplot(rf2,train.data$class,pallette=colors,main=paste("RF (unsupervised, ppc best 1000 features",
                                                        rf2$err.rate[500,1],")",sep=""))
  text(cords$points[,1],cords$points[,2],lab=as.numeric(patient.ids))
  rf3<-randomForest(t(aggregated.matrix)[,sample(1:nrow(aggregated.matrix),500)],proximity=TRUE)
## varImpPlot(rf3)
  if(seperate.output) {dev.off(); pdf(paste(output.dir,"/rf_mds3.pdf",sep=""))}  
  cords<-MDSplot(rf3,train.data$class,pallette=colors,main=paste("RF (unsupervised, random 1000 features",
                                                        rf3$err.rate[500,1],")",sep=""))
  text(cords$points[,1],cords$points[,2],lab=as.numeric(patient.ids))
  dev.off()  
} 

