####
## plot.hist.r
## original code from PPC R package
## modified by Janusz
####


plot.hist <-  function( data,cl.lab,class.vect, first.site=1, last.site=25, title.plot=NULL,ppc.split=NULL, type="default",
                       n.breaks=10,colors=c("red","lightgray"), test.name="", test.val =rep("",nrow(cl.lab)),ret.lab=FALSE){

  # first.site and last.site  indicate how many peak  sites to plot.
  # default is 1 thru 25
  
  old.par <- par(no.readonly = TRUE)
  on.exit(par(old.par))
  
  require(class)
  last.site<-min(last.site,dim(data)[1])
  
  sitelist <- first.site:last.site
  
  ytr<- as.factor(class.vect)
  codesy <- levels(ytr)
  
  nc<- length(codesy)
  nrows <- trunc(20/nc)
  
  par(mfcol=c(nrows,5))
  
  if(!is.null(title.plot)){ par(oma=c(0,0,5,0))}
  par(mar=c(0,1,0,1))
  par(cex=.6)
  ht <- data
  
  
  for(i in sitelist){
  
   xmax <- max(ht[i,])
   xmin <- min(ht[i,])
        
   ## Ensuring that the breaks in each histogram are same for all classes
   br <- hist(ht[i,],col=3,xlab="",axes=F,plot=F, main="", breaks=n.breaks, freq=FALSE)$br
   if (br[1]==0) br<-c((br[1]-diff(br)[1]),br)
    
  ## First histogram
    par(mar=c(0,1,1,1))
    junk <- hist(ht[i,ytr==codesy[1]],col=3,xlab="",axes=F,plot=F,breaks=br, freq=FALSE)
    hist(ht[i,ytr==codesy[1]],col=colors[1],axes=F,breaks=br, main="", freq=FALSE)
    lines(density(ht[i,ytr==codesy[1]]),adjust=.5) 
    box(col=gray(.5))
    
    lab<- paste("m/z",round(cl.lab[i,1],2))
    h <- max(junk$density)
    if (!ret.lab){ text((xmin+.7*(xmax-xmin)),.9*h,labels=lab,cex=.7)}
    else { lab = paste(lab, " ret", round(cl.lab[i,2],2))
          text((xmin+.5*(xmax-xmin)),.9*h,labels=lab,cex=.7)
   }         
    if (type=="ppc"){    
      v <- ppc.split[i]
      abline(v=v,lty=1,col=4)
      pr <- sum(ht[i,ytr==codesy[1]]>ppc.split[i])/sum(ytr==codesy[1])
      text((xmin+.9*(xmax-xmin)),h*.4,labels=as.character(round(pr,2)),col=1,cex=.7)
      pr2 <- sum(ht[i,ytr==codesy[2]]>ppc.split[i])/sum(ytr==codesy[2])
      lab<- paste(test.name, abs(round(pr,2)-round(pr2,2)))
      text((xmin+.8*(xmax-xmin)),.65*h,labels=lab,cex=.7)       
    }else{    
      lab<- paste(test.name, abs(round(test.val[i],2)))
      text((xmin+.8*(xmax-xmin)),.65*h,labels=lab,cex=.7)
  }  

  ## Second histogram

    par(mar=c(1,1,0,1))
    junk2 <- hist(ht[i,ytr==codesy[2]],col=4,xlab="",axes=F,plot=F,breaks=br, freq=FALSE)
    hist(ht[i,ytr==codesy[2]],col=colors[2],axes=F,breaks=br,main="", freq=FALSE) 
    lines(density(ht[i,ytr==codesy[2]],adjust=.5)) 
    box(col=gray(.5))
    h2 <- max(junk2$density)
    if (type=="ppc"){    
      abline(v=v,lty=1,col=4)
      text(v,.7*h2,labels=as.character(round(v,2)),col=3, cex=.6)
      text((xmin+.9*(xmax-xmin)),h2*.4,labels=as.character(round(pr2,2)),col=1,cex=.7)
    }
 }
  
 if(!is.null(title.plot)){ mtext(title.plot,outer=TRUE,side=3)} 
 return()
}

