## extensions to the xcms package
## model-based clusteringg
##
## author:  BOGUSLAW KLUGE, b.kluge@mimuw.edu.pl
##          MARTA LUKSZA, marta.luksza@molgen.mpg.de

RANDOM.FOREST.IMP.PROGRAM <- '~/Polen/xcmsMBC/randomForest/rf_imp.out'
RANDOM.FOREST.TMP.DIR <- '~/Polen/xcmsMBC/randomForest/tmp.dir'

default.eps <- list(mean=1e-6, sd=1e-6, proportions=1e-6)

setClass("xcmsSetWrapper",
         representation(set="xcmsSet",
                        profinfo="list",
                        clustinfo="list"
                        )
         
         )

xcmsSetWrapper <- function(set){
  object <- new("xcmsSetWrapper")
  object@set <- set
  object@profinfo <- list()
  object@clustinfo <- list()
  return(object)
}


xcmsSetCopy <- function(set){
    object <- new("xcmsSet")
    peaks(object) <- peaks(set)
    groups(object) <- groups(set)
    groupidx(object) <- groupidx(set)
   sampnames(object) <- sampnames(set)
    sampclass(object) <- sampclass(set)
    object@rt <- set@rt
    object@cdfpaths <- set@cdfpaths
    objectprofinfo <- set@profinfo
    return(object)
}

FilterClusters <- function(s,minProb){
  c <- s$cl
  for(i in 1:nrow(s$ba)){
    if(s$ba[i,"probability"]<minProb)
      c[i] <- 0
  }
  return(c)
}

FilterClusters.prev <- function(s,minProb){
  c <- s$cl
  mc <- s$cl
  for(i in 1:nrow(s$z)){
    mc[i] <- max(s$z[i,])
    if(mc[i]<minProb)
      c[i] <- 0
  }
  return(c)
}

InitializeClusters <- function(points,
                               sd,
                               sd.modes,
                               hcPairs,
                               G){
  N <- nrow(points)
  ncol <- ncol(points)
  logN <- log(N)
  clss <- hclass(hcPairs, G)
  clusters <- list()
  
  clusters$mean <- matrix(0,nrow=G,ncol=ncol)
  clusters$sd <- matrix(0,nrow=G,ncol=ncol)
  clusters$log.proportions <- rep(0, G)
  clu <- unique(clss)
  ridx <- 1:N
  for(cl in clu){
    idx <- which(clss==cl)
    n <- length(idx)
    spoints <- subset.matrix(points, ridx %in% idx)


    sds <- rep(0,ncol)
  
    for(i in seq(along=idx)){
      sds <- sds+spoints[i,]^2
    }
    sds <- sds/n
    clusters$sd[cl,] <- sds
    
    clusters$mean[cl,] <- colMeans(spoints)
    clusters$log.proportions[cl] <- log(n) - logN
  }

  for(j in 1:ncol){
    if(sd.modes[j]==1)
      clusters$sd[,j] <- sd[j]
    else if(sd.modes[j]==3)
      clusters$sd[,j] <- mean(clusters$sd[,j])
  }
  return(clusters)
}


bicC <- function(loglik, n, d, G, sd.modes)
{
  nparams <- G*d ##cluster means
  for(sd.mode in sd.modes){
    if (sd.mode==2)
      nparams <- nparams + G
    else if (sd.mode==3)
      nparams <- nparams + 1
  }
  bic <- 2 * loglik - nparams * log(n)
  return(bic)
}


CMclust <- function(peakdata,
                    sd,
                    sd.modes,
                    G,
                    hcPairs,
                    eps=default.eps){
  best.bic <- -Inf
  best.cl <- NULL
  n <- nrow(peakdata)
  d <- ncol(peakdata)
  print(G)
  for(k in G){
    print(paste(k,"in CMclust"))
    cl <- InitializeClusters(peakdata, sd, sd.modes,
                             hcPairs, k)
    ## loglik <- log.likelihood(peakdata, cl)
    ##print(paste("loglik before:",loglik))
    hyper.params <- generate.hyper.params(points=peakdata, clusters.num=k)

    cl <- next.clusters.until.converged(points=peakdata,
                                        cl=cl,
                                        sd.modes=sd.modes,
                                        hyper.params=hyper.params,
                                        eps=eps)
    ## loglik <- log.likelihood(peakdata, cl)
    loglik <- log.posterior.unnorm(points=peakdata,
                                   clusters=cl,
                                   sd.modes=sd.modes,
                                   hyper.params=hyper.params)
    if(is.na(loglik))
      return(cl)
    print(paste("loglik after:",loglik))
    if(!is.na(loglik)){
      bic <- bicC(loglik, n, d, G=k, sd.modes)
      print(bic)
      if (best.bic < bic){
        best.bic <- bic
        best.cl <- cl
      }
    }
  }
  summary <- NULL
  if(!is.null(best.cl)){
    summary <- list()
    summary$ba <- best.assignment(peakdata,best.cl)
    summary$cl <- summary$ba[,"cluster"]
    summary$bic <- best.bic
    summary$uncertainty <- 0
  }
  return(summary)
}



if( !isGeneric("MBC") )
    setGeneric("MBC", function(object, ...) standardGeneric("MBC"))

setMethod("MBC", "xcmsSetWrapper", function(object, mz=TRUE, eps = c(1,10),
                                            minPts = 0,
                                            maxsize = -1, minsize = 0,
                                            mznorm =1, rtnorm = 1,
                                            modelName = "C",
                                            eps.em = default.eps,
                                            mzstdev=0.01, rtstdev=80,
                                            sd.modes=c(1,1),
                                            minProb = 0, mulclust = 4,preclust) {
  N <- nrow(peaks(object@set))
  object@profinfo$mz <- mz
  if(missing(preclust)){
    for(i in seq(along=eps)){
      if (i==1) pref <- ""
      else pref <- "_"
      object@profinfo$eps <- paste(pref,eps,sep="")
    }
    object@profinfo$minPts <- minPts
  }
     
  else{
    object@profinfo$minPts <- preclust$minPts
    object@profinfo$eps <- paste(preclust$epsmz, preclust$epsrt, sep="_")
  }
  object@profinfo$maxsize <- maxsize
  object@profinfo$modelName <- modelName
  if(modelName=="C"){
    object@profinfo$mzstdev <- mzstdev
    object@profinfo$rtstdev <- rtstdev
    object@profinfo$sd.modes <- sd.modes
  }
  object@profinfo$rtnorm <- rtnorm
  object@profinfo$minProb <- minProb
 
  
  if(missing(preclust))
    preclust <- DBSCAN(object,mz=mz,eps=eps,minPts=minPts,maxsize=maxsize, set=FALSE)
  print("preliminary clustering done.")
  
  clust <- rep(-1,length(preclust$cl))
          
  tab <- table(preclust$cl)
  
  cllist <- as.numeric(names(tab[which(tab[]>minsize)]))
  ccllist <- as.numeric(names(tab[which(tab[]<=minsize)]))

  for(i in seq(along=ccllist)){
    idx <- which(preclust$cl==ccllist[i])
    clust[idx] <- i
  }
#  cllist <- unique(preclust[which(preclust!=0)])
  cllist <- unique(cllist[which(cllist!=0)])

  object@clustinfo$bics <- double(length(cllist))
  object@clustinfo$uncertainties <- double(length(cllist))
  
  print(paste("number of preliminary clusters: ",length(cllist)))
  per <- length(cllist)/100

  counter <- round(per)
  mul <- 0

  sampnum <- length(sampnames(object@set))
  save(cllist,file=".RData_out")
  for(i in seq(along=cllist)){
    cl <- cllist[i]
    idx <- which(preclust$cl==cl)
    n <- length(idx)
    counter <- counter-1
    if(counter==0){
      counter <- round(per)
      mul <- mul+1
      print(paste(i,": ",mul," %, ",length(idx)))
    }
    
    di <- "mass"
    if (mz)
      di <- "mz"
    peakdatao <- subset.matrix(peaks(object@set), 1:N %in% idx)
    peakdatao <- peakdatao[,c(di,"rt"),drop=FALSE]
    peakdata <- peakdatao

    peakdata[,di] <- peakdata[,di]/mznorm 
    peakdata[,"rt"] <- peakdata[,"rt"]/rtnorm
    maks <- max(c(0,clust))

    if(nrow(peakdata)>1){
      hcp <- hc(peakdata, modelName="EII")
      minG <- floor(nrow(peakdatao)/sampnum)
      if(minG == 0)
        minG <- minG+1
      maxG <- min(minG*mulclust,n)
      write(paste("processing cluster",i,"(",cl,")","with",n,"elements"),file=paste("out_",modelName,sep="")) 
      if(modelName=="C"){
        clustm <- CMclust(peakdatao,
                          sd=c(mzstdev,rtstdev),
                          sd.modes=sd.modes,
                          G=minG:maxG,
                          hcPairs=hcp,
                          eps=eps.em
                          )
      }
      else{
        if(sd(peakdatao[,1])==0)
          peakdatao[,1] <- peakdatao[,1]+rnorm(n,0,1e-3)
        if(sd(peakdatao[,2])==0)
          peakdatao[,2] <- peakdatao[,2]+rnorm(n,0,1e-3)
        clustm <- try(Mclust(peakdatao,modelNames=c(modelName),
                         G=minG:maxG,initialization=list(hcPairs=hcp)))
        if(is(clustm)=="try-error"){
	  print(paste("mclust failed with given hcp, going for default"))
          clustm <- Mclust(peakdatao,modelNames=c(modelName),G=1:max(maxG,9))
	}	
        ba <- matrix(rep(0,2*n),ncol=2)
        colnames(ba) <- c("cluster","probability")
        if(is.null(clustm$z))
          ba[1:n,] <-c(1,1)
        
        else
          for(i in 1:n){
            idmax <- which.max(clustm$z[i,])
            ba[i,] <- c(idmax,clustm$z[i,idmax])
          }
        clustm$ba <- ba
      }
      if(!is.null(clustm)){
        clustm$cl <- FilterClusters(clustm,minProb)
        if(is.double(clustm$bic)){
          ##      if(!(is.na(clustm$bic) || is.infinite(clustm$bic)))
          object@clustinfo$bics[i] <- clustm$bic
                                        #      if(!(is.na(clustm$uncertainty) || is.infinite(clustm$uncertainty)))
          object@clustinfo$uncertainties[i] <- mean(clustm$uncertainty)
          
        }
        else {
          object@clustinfo$bics[i] <- 100000
          object@clustinfo$uncertainties[i] <- 1
        }      
        clustin <- rep(0,length(clustm$cl))
        nzeroidx <- which(clustm$cl!=0)
        clustin[nzeroidx] <- clustm$cl[nzeroidx]+maks
      }
      
      else if (cl > 0) 
        clustin <- c(cl)+maks

   
      clust[idx] <- clustin
    }
  }
  MBC2(object,clust)  
})


if( !isGeneric("MBC2") )
    setGeneric("MBC2", function(object, ...) standardGeneric("MBC2"))

setMethod("MBC2", "xcmsSetWrapper", function(object, clust, maxsize = -1, minsize=0) {
  samples <- sampnames(object@set)
  classlabel <- sampclass(object@set)
  classnames <- levels(classlabel)
  classlabel <- as.vector(unclass(classlabel))
  classnum <- integer(max(classlabel))
  for (i in seq(along = classnum))
    classnum[i] <- sum(classlabel == i)
  
  gcount <- integer(length(classnum))

  tab <- table(clust)
  idx <- which(tab[]>minsize)
  cl <- as.numeric(names(tab[idx]))
  cl <- cl[which(cl>0)]
 
  idx <- c()

  nclust <- length(cl)

  groupmat <- matrix(nrow = nclust, ncol = 7 + length(classnum))
  groupindex <- vector("list", nclust)
  print(paste("gathering info about clusters...",nclust))
  tper <- round(length(cl)/100)
  counter <- tper
  mul <- 0

  peakmat <- peaks(object@set)

  for (num in seq(along=cl)){
    counter <- counter-1
    if(counter==0){
      counter <- tper
      mul <- mul+1
      print(paste(mul,"%"))
    }
    clnum <- cl[num]

    idx <- which(clust==clnum)

    speakmat <- peakmat[idx,]

    if (length(idx)==1){
      gnum <- classlabel[speakmat["sample"]]    
    
      groupmat[num, 1] <- median(speakmat["mz"])
      groupmat[num, 2:3] <- range(speakmat["mz"])
      groupmat[num, 4] <- median(speakmat["rt"])
      groupmat[num, 5:6] <- range(speakmat["rt"])      
    }
    else{
      gnum <- classlabel[unique(speakmat[,"sample"])]
    
      groupmat[num, 1] <- median(speakmat[, "mz"])
      groupmat[num, 2:3] <- range(speakmat[, "mz"])
      groupmat[num, 4] <- median(speakmat[, "rt"])
      groupmat[num, 5:6] <- range(speakmat[, "rt"])
    }

    for (j in seq(along = gcount))
      gcount[j] <- sum(gnum == j)

    groupmat[num, 7] <- length(idx)
    groupmat[num, 7+seq(along = gcount)] <- gcount
    
    groupindex[[num]] <- sort(idx)
    
  }
 
  cat("\n")
  
  colnames(groupmat) <- c("mzmed", "mzmin", "mzmax", "rtmed", "rtmin", "rtmax",
                            "npeaks", classnames)
    
  groups(object@set) <- groupmat[1:nclust,]
  groupidx(object@set) <- groupindex[1:nclust]
  object
})

# z metryka supremum
dbscan <- function(points, eps, minPts) {
  points <- as.matrix(points)
  if(!is.matrix(points))
    stop("The argument must be a numeric matrix.")
  if(ncol(points) > 2)
    stop("Currently only dim 1 and 2 supported.")
  if(length(eps) != ncol(points))
    stop("length(eps) must be equal to ncol(points).")
  for(i in seq(along=eps))
    print(eps[i])
  for(k in 1:ncol(points))
    points[,k] <- points[,k] / eps[k]
   
  res <- rep(-1, nrow(points))
  .C(c("wrapper1D", "wrapper2D")[ncol(points)],
     as.double(points),
     as.integer(nrow(points)),
     as.double(1),
     as.integer(minPts),
     res=as.double(res))$res
}


if( !isGeneric("DBSCAN") )
    setGeneric("DBSCAN", function(object, ...) standardGeneric("DBSCAN"))

setMethod("DBSCAN", "xcmsSetWrapper", function(object, mz=TRUE, eps = c(1,10), minPts = 0, maxsize = -1, set=FALSE) {

  samples <- sampnames(object@set)
  classlabel <- sampclass(object@set)
  classnames <- levels(classlabel)
  classlabel <- as.vector(unclass(classlabel))
  classnum <- integer(max(classlabel))
  for (i in seq(along = classnum))
    classnum[i] <- sum(classlabel == i)
  
  peakmat <- peaks(object@set)
  idx <- 1:nrow(peakmat)
  
  gcount <- integer(length(classnum))

  ob <- list()
  ob$idx <- idx
  ob$mul <- 1
  queue <- list()
  queue[[1]] <- ob

  clust <- rep(-1, nrow(peakmat))
  di <- "mass"
  if(mz)
    di <- "mz"
  while(length(queue)>0){
    ob <- queue[[1]]
    idx <- ob$idx
    mul <- ob$mul
    print(paste("next: ",range(idx)))
    peakdata <- peakmat[idx,c(di,"rt")]
    if (length(queue) ==1) queue <- list()
    else queue <- queue[2:length(queue)]
    peakdata <- peakmat[idx,c(di,"rt")]

    clustin <- dbscan(peakdata, eps*mul, minPts)
    
    maks <- max(c(clust,0))
    for(i in seq(along=clustin)){
      if(clustin[i] != 0)
        clustin[i] <- clustin[i]+maks
    }
    clust[idx] <- clustin 
    if (maxsize != -1){ #the maximum cluster size is set
      minPts <- 0
      tab <- table(clustin)
      bigcl <- which(tab[]>maxsize & as.numeric(names(tab))!=0)

      for (i in seq(along=bigcl)){
        clnum <- as.numeric(names(tab)[bigcl[i]])
        ##          if (clnum != 0){#if not the noise cluster
            
        idx1 <- which(clustin==clnum)
        idx2 <- idx[idx1]
        ob2 <- list()
        ob2$idx <- idx2
        ob2$mul <- mul*0.9
        queue[[length(queue)+1]] <- ob2
      }

    }
    print(paste("queue size: ",length(queue)))
  }
  if (set){
    object <- MBC2(object, clust)
    return(object)
  }
  else return(list(cl=clust,epsmz=eps[1],epsrt=eps[2],minPts=minPts))
})


if( !isGeneric("groupAssignments") )
    setGeneric("groupAssignments", function(object, ...) standardGeneric("groupAssignments"))

setMethod("groupAssignments", "xcmsSetWrapper", function(object){
  gidx <- groupidx(object@set)
  ass <- rep(0, times=nrow(peaks(object@set)))
  for(i in 1:length(gidx)){
    ass[gidx[[i]]] <- i
  }
  ass
  
})

if( !isGeneric("groupAssignmentsFiltered") )
    setGeneric("groupAssignmentsFiltered", function(object, ...) standardGeneric("groupAssignmentsFiltered"))

setMethod("groupAssignmentsFiltered", "xcmsSetWrapper", function(object,
                                                                 minsize=0,
                                                                 minfrac = 0.5,
                                                                 minsamp = 1, retgood=FALSE){
  
  gidx <- groupidx(object@set)
  ass <- rep(0, times=nrow(peaks(object@set)))
  sclass <- sampclass(object@set)
  levels <- levels(sclass)
  classlabel <- as.vector(unclass(sclass))

  peaks <- peaks(object@set)
  classnum <- integer(max(classlabel))
  
  for (i in seq(along = classnum))
    classnum[i] <- sum(classlabel == i)

  nsamp <- length(sampnames)

  gcount <- integer(length(classnum))

  good <- c()
  for(i in 1:length(gidx)){
#   
    speakmat <- peaks[groupidx(object@set)[[i]],,drop=FALSE]
    
    gnum <- classlabel[unique(speakmat[,"sample"])]
    for (j in seq(along = gcount))
      gcount[j] <- sum(which(gnum == j))
    if (! any(gcount >= classnum*minfrac & gcount >= minsamp)){
      next
    }
    if (sum(gcount)<minsize)
      next
    else{
      ass[gidx[[i]]] <- i
      good <- append(good,i)
    }
    
  }
  if(retgood)
    return(good)
 else return(ass)
  
})

if( !isGeneric("FDR") )
    setGeneric("FDR", function(object, ...) standardGeneric("FDR"))

setMethod("FDR", "xcmsSetWrapper", function(object, nperms=10,
                                            fdr, tseq,
                                            minsamp=0,minfrac=0,minsize=0,
                                            bin,test="t.equalvar"){
  ##  source("/home/mluksza/xcms/R/miner/include.R")

  if(missing(fdr)){
    print("permutations")
    ass <- groupAssignmentsFiltered(object,minfrac=minfrac,minsamp=minsamp,
                                    minsize=minsize,retgood=TRUE)
    idx <- which(ass!=0)
    ass <- ass[idx]
    ngroups <- nrow(groups(object@set))
    gv <- groupval(object@set)
    gv[which(is.na(gv))] <- 0
    gv <- gv[ass,]
    sclass <- unmap(sampclass(object@set))
    N <- nrow(sclass)
    classlabel <- rep(0,N)
    
    for(i in 1:N)
      classlabel[i] <- which(sclass[i,]==1)-1 ##sclass[[i]]-1
   
    if(missing(test) || test == "t" || test == "t.equalvar"){
#      require(multtest) || stop("Couldn't load multtest")
      
      to <- mt.teststat(gv,classlabel=classlabel)
      control <- matrix(0,ncol=nperms,nrow=nrow(gv))
      cl <- classlabel
      for (i in 1:nperms){
        print(paste("permutation",i))
        cl <- sample(cl)
        control[,i] <- mt.teststat(gv,classlabel=cl,test=test)
      }
      fdr <- list()
      fdr$true.score <- abs(to)
      fdr$perm.scores <- abs(control)
    }
    else{
      fdr <- compute.fdr(gv,as.vector(sclass),nperms=nperms,test=test)
    }
  }
  ##compute statistics

  res <- report.fdr(fdr)
  return(res)
})


if( !isGeneric("RFClassifier") )
  setGeneric("RFClassifier", function(object, ...) standardGeneric("RFClassifier"))

setMethod("RFClassifier", "xcmsSetWrapper", function(object,
                                                     minsamp=1,minfrac=0.5){

  
  ass <- groupAssignmentsFiltered(object,minfrac=minfrac,minsamp=minsamp,retgood=TRUE)
  idx <- which(ass!=0)
  ass <- ass[idx]
  ngroups <- nrow(groups(object@set))
  gv <- groupval(object@set)
  gv[which(is.na(gv))] <- 0
  gv <- gv[ass,]
  sclass <- unmap(sampclass(object@set))
  N <- nrow(sclass)
  sclass <- as.vector(sclass)
  classlabel <- sclass

  
  
})

if( !isGeneric("trainTable") )
    setGeneric("trainTable", function(object, ...) standardGeneric("trainTable"))

setMethod("trainTable", "xcmsSetWrapper", function(object,
                                                   data.train="data.train",
                                                   train.path="train.csv",
                                                   cluster.info.path="clusterinfo.csv",
                                                   minsize=0, minfrac = 0.5,
                                                   minsamp = 1,paramsfile=FALSE,jbt=1000){

  minPts <- object@profinfo$minPts
  maxsize <- object@profinfo$maxsize
  eps <- object@profinfo$eps
  mzstdev <- object@profinfo$mzstdev
  rtstdev <- object@profinfo$rtstdev
  rtnorm <- object@profinfo$rtnorm
  minprob <- object@profinfo$minProb
  modelName <- object@profinfo$modelName
  
  str <- paste("db_",eps,"_",modelName,sep="")
  
  if(modelName == "C"){
    ssd <- paste("_sd_",paste(object@profinfo$sd.modes,collapse="_"),sep="")
    str <- paste(str,ssd,sep="")
    if(!any(object@profinfo$sd.modes!=1)){
      str <- paste(str,"_",rtstdev,sep="")

    }     
  }
  data.train <- paste("data_",str,".train",sep="")
  cluster.info.path <- paste("cinfo_",str,".csv",sep="")
  groups <- groups(object@set)
  peaks <- peaks(object@set)
  ngroups <- nrow(groups)
  sampnames <- sampnames(object@set)
  
  sclass <- sampclass(object@set)
  levels <- levels(sclass)
  classlabel <- as.vector(unclass(sclass))
  
  classnum <- integer(max(classlabel))
  nsamp <- length(sampnames)
  groupval <- groupval(object@set)

  gcount <- integer(length(classnum))
  
  train.file <- matrix(c(1:nsamp),ncol=1)
  train.file <- cbind(train.file, sampnames)
  lab <- c()
  labn <- c()
  
  for (i in seq(along=sclass)){
    lab <- append(lab, levels[sclass[[i]]])
    labn <- append(labn,sclass[[i]])
  }
  
  train.file <- cbind(train.file, lab)
  colnames(train.file) = c("id", "file","class")

  write.table(train.file, file=train.path, row.names=FALSE, sep = '\t', quote=FALSE)
  
  gass <- groupAssignmentsFiltered(object,minfrac=minfrac,
                                   minsamp=minsamp,
                                   minsize=minsize,
                                   retgood=TRUE)
  gass <- gass[which(gass!=0)]
  
  minfo <- matrix(ncol=3)
  m <- matrix(nrow=0,ncol=nsamp)

  for (i in seq(along=gass)){
    gr <- gass[i]
    
    speakmat <- peaks[groupidx(object@set)[[gr]],,drop=FALSE]
       
    obs <- peaks[groupval[gr,],][,"maxo"]
    g <- groups[gr,]
    
    info <- c(i, g["mzmed"], g["rtmed"])
    m <- rbind(m,obs)
    minfo <- rbind(minfo, info)
    
  }
#  m <- matrix(m,byrow=TRUE, ncol=nsamp)

  classes <- c()

  m <- round(m,3)
  colnames(m) <- sampnames
  rownames(m) <- 1:nrow(m)
  m <- rbind(m,labn)
  
  minfo <- round(minfo, 3)
  colnames(minfo) <- c("cluster.id", "mzmed", "rtmed")

  m[which(is.na(m))] <- 0
  mm <- t(m)
  write.table(mm, file=data.train,col.names=F, row.names=FALSE, sep = ' ', quote=FALSE)
  
  write.table(minfo, file=cluster.info.path, row.names=FALSE, sep = '\t', quote=FALSE)
  if(paramsfile){
    
    nobs <- nrow(mm)
    nvar <- (ncol(mm) - 1)
    mtry0 <- round(sqrt(nvar))
    
    cat(paste('"',data.train,'"\n','"',"importance",str,'"\n',nvar,"\n",nobs,"\n",mtry0,"\n",jbt,"\n","0\n",sep=""),
        file=paste("params_",str,".txt",sep=""))
    
  }
})



#extension: peaks.mass
if( !isGeneric("readMasses") )
    setGeneric("readMasses", function(object, ...) standardGeneric("readMasses"))

setMethod("readMasses", "xcmsSetWrapper", function(object, paths, ext = "peaks.mass"){

  cnames <- c("mass","mz","mzmin","mzmax","rt","rtmin","rtmax","into","intf", "maxo","maxf","charge", "sample")

  peaks <- matrix(nrow = 0, ncol = 13)
  
  colnames(peaks) <- cnames
  
  names <- strsplit(paths,"/")
  counter <- 0
  for(i in seq(along=paths)){
    len <- length(names[[i]])
    sname <- names[[i]][len]
    sn <- strsplit(sname,".",fixed=TRUE)
    sname <- unlist(sn)[1]
#    sample <- which(sampnames(object)==sname)

   #print(sample)
#    if(length(sample)>0){
#      counter <- counter+1
    print(sname)
    tab <- read.table(paths[i],header=TRUE,sep="\t")
    mass <- as.double(tab[,"Mass"])
    mz <- as.double(tab[,"mz"])
    rt <- as.double(tab[,"time"]*60)
    o <- as.double(tab[,"first_vol"])
    f <- as.double(tab[,"vol"])
    ch <- as.integer(tab[,"charge"])
    sampcol <- rep(sample,nrow(tab))
    peaks1 <- matrix(c(mass,mz,mz,mz,rt,rt,rt,f,o,f,o, ch, sampcol),ncol=13)
    peaks <- rbind(peaks,peaks1)
                                        #    }
  }
  peaks(object@set) <- peaks
  print(counter)
  object
})



##number of masses per sample
if( !isGeneric("histogram0") )
    setGeneric("histogram0", function(object, ...) standardGeneric("histogram0"))

setMethod("histogram0", "xcmsSetWrapper", function(object){

  peaks <- peaks(object@set)
  pno <- nrow(peaks)
  sampshisto <- integer(length(sampnames(object@set)))
  
  for(i in seq(along=sampnames(object@set))){
    pno <- length(which(peaks[,"sample"]==i))
    sampshisto[i] <- pno
  }
  return(sampshisto)

})


##number of clusters per sample
if( !isGeneric("histogram1") )
  setGeneric("histogram1", function(object, ...) standardGeneric("histogram1"))

setMethod("histogram1", "xcmsSetWrapper", function(object,minsamp=1,minfrac=0.5){
  gv <- groupval(object@set)
  gidx <- groupidx(object@set)
  gass <- groupAssignmentsFiltered(object,minsamp=minsamp,minfrac=minfrac,retgood=TRUE)
  gass <- gass[which(gass!=0)]
  peaks <- peaks(object@set)
  
  sampshisto <- integer(length(sampnames(object@set)))
  
  for(i in seq(along=gass)){
    gr <- gass[i]
    g <- gidx[[gr]]
    samples <- unique(peaks[g,"sample"])
    sampshisto[samples] <- sampshisto[samples]+1
  }
  return(sampshisto)
})

##number of samples per cluster
if( !isGeneric("histogram2") )
  setGeneric("histogram2", function(object, ...) standardGeneric("histogram2"))

setMethod("histogram2", "xcmsSetWrapper", function(object,minfrac=0.5,minsamp=1){
  gidx <- groupidx(object@set)
  peaks <- peaks(object@set)
  
  sclass <- sampclass(object@set)
  levels <- levels(sclass)
  classlabel <- as.vector(unclass(sclass))
  classnum <- integer(max(classlabel))
  gcount <- integer(length(classnum))

  gass <- groupAssignmentsFiltered(object,minsamp=minsamp,minfrac=minfrac,retgood=TRUE)
  gass <- gass[which(gass!=0)]
  sampshisto <- integer(length(gass))

  levels <- levels(sclass)
  sampshisto1 <- matrix(nrow=length(gass),ncol=(length(levels)+1) )
  colnames(sampshisto1) <- c(levels,"all")
  gcount <- integer(length(levels)+1)
  for(i in seq(along=gass)){
    gr <- gass[i]
    g <- gidx[[gr]]
    sampshisto[i] <- length(unique(peaks[g,"sample"]))
#    gnum <- sclass[peaks[g,"sample"]]
    gnum <- classlabel[peaks[g,"sample"]]
    for (j in seq(along = gcount))
      gcount[j] <- sum(gnum == j)

    
    
#    for(j in seq(along = levels)){
#      gcount[j] <- sum(gnum==levels[j])
#    }
    gcount[length(levels)+1] <- sum(gcount[1:length(levels)])
    sampshisto1[i,] <- gcount
  }

  return(list(opt=sampshisto, detail=sampshisto1))

})


##number of clusters per samples
if( !isGeneric("histogram3") )
  setGeneric("histogram3", function(object, ...) standardGeneric("histogram3"))

setMethod("histogram3", "xcmsSetWrapper", function(object,minfrac=0.5,minsamp=1){
  gidx <- groupidx(object@set)  
  samples <- object@set@peaks[,"sample"]

  d <- c()

  process <- function(g, samples){
    t <- as.vector(table(samples[g]))
    return(t)
  }
  x <- sapply(gidx,FUN=process,samples=samples)
  l <- sapply(gidx,length)
  x <- x[which(l>=minsamp)]
  x <- unlist(x)
  return(x)
})

          
if( !isGeneric("group") )
  setGeneric("group", function(object, ...) standardGeneric("group"))

setMethod("group", "xcmsSetWrapper", function(object, ...){
  object@set <- group(object@set,...)
  return(object)
})

if( !isGeneric("retcor") )
  setGeneric("retcor", function(object, ...) standardGeneric("retcor"))

setMethod("retcor", "xcmsSetWrapper", function(object, ...){
  object@set <- retcor(object@set,...)
  return(object)
})


