#ifndef _DBSCANCLUSTRATOR_H_
#define _DBSCANCLUSTRATOR_H_

#include <deque>
#include <vector>
#include "kdtree++/kdtree.hpp"

using namespace std;

template <int N> class DBSCANClustrator
{
public:
	DBSCANClustrator(){};
	virtual ~DBSCANClustrator(){};

	private:
	
		struct point_t {
			typedef double value_type;
  			int id;
  			double p[N];
  			bool classified;
  			inline double operator[](size_t const k) const { return p[k]; }
		};
	
		typedef KDTree::KDTree<N, point_t> tree_t;
		typedef typename tree_t::const_iterator it_t;
		tree_t tree;
	
	public:
		void dbscan(double *data, int n, double eps, int minPts, double *res)
		{
			for(int k=0; k<n; ++k) {
		  		point_t p;
	  			p.id = k;
				res[k] = -1;
			for(int d=0; d<N; ++d)
			    p.p[d] = data[d*n+k];
				tree.insert(p);
			}

			int clusterId = 1;
			
			bool expanded = false;
			
			for(it_t ip = tree.begin(); ip != tree.end(); ++ip){
	  			if (res[ip->id] != -1) 
				    continue;
				cout<<ip->id<<endl;
				expanded = ExpandCluster(*ip, clusterId, eps, minPts, res);
				if (expanded)
			    	clusterId++;
			}
		
		
		};
		
	protected:
		bool ExpandCluster(point_t p, int clusterId, double eps, int MinPts, double *res)
		{
		
			vector<point_t> v;
			
			deque<point_t> seeds;
			
			tree.find_within_range(p, eps, back_inserter(v));
			int size = (int)v.size();
			if (size < MinPts){
				res[p.id] = 0;
				return false;
			}
			res[p.id] = clusterId;
			
			for(int k=0; k<(int)v.size(); ++k){ 
				res[v[k].id] = clusterId;
				seeds.push_back(v[k]);				
			}
			
			v.erase(v.begin(),v.end());
			while(seeds.size()>0)
			{
				point_t f = seeds.front();
				seeds.pop_front();

				tree.find_within_range(p, eps, back_inserter(v));

				size = (int)v.size();
				if (size >= MinPts){
					for(int k=0; k<(int)v.size(); ++k){ 
						point_t p2 = v[k];	
						if (res[p2.id] < 1) //unclassified or noise
						{
							if (res[p2.id] == -1) //unclassified
								seeds.push_back(p2);
										
							res[p2.id] = clusterId;			
						}
					}
				}
			}
										
			return true;
		}
};

#endif //_DBSCANCLUSTRATOR_H_
