#include <iostream>
#include <deque>
#include <vector>
#include <set>

#include "kdtree++/kdtree.hpp"

using namespace std;


template<int N> struct point_t {
	typedef double value_type;
	int id;
	double p[N];
	inline double operator[](size_t const k) const { return p[k]; }
};

int expand_cluster(int prevClust, int v, int minPts, int t,
		    const vector< vector<int> > &graph, double *res) {

  deque<int> q;

  q.push_back(v);
  int u, w;
  int prev = prevClust;
  while(!q.empty()){

    u = q.front();
    q.pop_front();
    if ((int)graph[u].size() < minPts){
      res[u] = prev; 
    }
    else { //cluster is expanded
      res[u] = t;
      prev = t;
      for(int k=0; k<(int)graph[u].size(); ++k){
	w = graph[u][k];
	if (res[w] == 0.0)
	  res[w] = t;
	else if (res[w] == -1.0){
	  res[w] = t;
	  q.push_back(w);
	}
      }
    }

  }//while
  return prev;

}

template <int N> void dbscan(double *data,
			     int n, double eps, int minPts, double *res) {
  typedef KDTree::KDTree< N, point_t<N> > tree_t;
  typedef typename tree_t::const_iterator it_t;
  tree_t tree;
  //  cout<<"starting DBSCAN"<<endl;
  for(int k=0; k<n; ++k) {
    point_t<N> p;
    p.id = k;
    for(int d=0; d<N; ++d)
      p.p[d] = data[d*n+k];
    tree.insert(p);
  }
  vector< vector<int> > graph(n);
  for(it_t p=tree.begin(); p!=tree.end(); ++p) {
    vector< point_t<N> > v;
    tree.find_within_range(*p, eps, back_inserter(v));
    for(int k=0; k<(int)v.size(); ++k) {
      graph[p->id].push_back(v[k].id);
      graph[v[k].id].push_back(p->id);
    }
  }
  for(int k=0; k<(int)graph.size(); ++k) {
    sort(graph[k].begin(), graph[k].end());
    graph[k].erase(unique(graph[k].begin(), graph[k].end()),
		   graph[k].end());
  }
  for(int k=0; k<n; ++k) res[k] = -1.0;
  int t=1;
  int expanded;

  for(int k=0; k<n; ++k)
    if(res[k] == -1.0){
      expanded = expand_cluster(0, k, minPts, t, graph, res);
      if (expanded == t) t++;
    }
  //  cout<<"DBSCAN succeded"<<endl; 
}

extern "C" {
  //n - liczba elementów (wierzy macierzy data)
	void wrapper1D(double *data, int *n, double *eps, int *minPts, double *res) {
		dbscan<1>(data, *n, *eps, *minPts, res);
	}
	
	void wrapper2D(double *data, int *n, double *eps, int *minPts, double *res) {
		dbscan<2>(data, *n, *eps, *minPts, res);
	}
}
